@@ -3,11 +3,9 @@
 from __future__ import annotations
 
 import asyncio
-from collections.abc import Callable, Coroutine, Sequence
+from collections.abc import Callable
 from contextlib import AsyncExitStack, asynccontextmanager
 from dataclasses import dataclass, field
-from datetime import datetime
-import os
 from os import PathLike
 import time
 from typing import TYPE_CHECKING, Any, Literal, Self, TypedDict, cast, overload
@@ -20,8 +18,7 @@ from typing_extensions import TypeVar
 from llmling_agent.log import get_logger
 from llmling_agent.messaging.messagenode import MessageNode
 from llmling_agent.messaging.messages import ChatMessage, TokenCost
-from llmling_agent.models.session import MemoryConfig, SessionQuery
-from llmling_agent.observability import track_action, track_agent
+from llmling_agent.observability import track_action
 from llmling_agent.prompts.builtin_provider import RuntimePromptProvider
 from llmling_agent.prompts.convert import convert_prompts
 from llmling_agent.resource_providers.runtime import RuntimeResourceProvider
@@ -33,17 +30,22 @@ from llmling_agent.utils.inspection import (
     has_return_type,
     validate_import,
 )
+from llmling_agent.utils.now import get_now
 from llmling_agent.utils.result_utils import to_type
 from llmling_agent.utils.tasks import TaskManagerMixin
+from llmling_agent_config.session import MemoryConfig
 
 
 if TYPE_CHECKING:
-    from collections.abc import AsyncIterator
+    from collections.abc import AsyncIterator, Coroutine, Sequence
+    from datetime import datetime
+    import os
     from types import TracebackType
 
     from llmling.config.models import Resource
     from llmling.prompts import PromptType
     import PIL.Image
+    from pydantic_ai.run import AgentRun
     from toprompt import AnyPromptType
 
     from llmling_agent.agent import AgentContext, AnyAgent
@@ -61,20 +63,20 @@ if TYPE_CHECKING:
     from llmling_agent.config.capabilities import Capabilities
     from llmling_agent.delegation.team import Team
     from llmling_agent.delegation.teamrun import TeamRun
-    from llmling_agent.models.mcp_server import MCPServerConfig
-    from llmling_agent.models.providers import ProcessorCallback
-    from llmling_agent.models.result_types import ResponseDefinition
-    from llmling_agent.models.task import Job
+    from llmling_agent_config.mcp_server import MCPServerConfig
+    from llmling_agent_config.providers import ProcessorCallback
+    from llmling_agent_config.result_types import StructuredResponseConfig
+    from llmling_agent_config.session import SessionQuery
+    from llmling_agent_config.task import Job
     from llmling_agent_input.base import InputProvider
-    from llmling_agent_providers.base import (
-        AgentProvider,
-        StreamingResponseProtocol,
-        UsageLimits,
-    )
+    from llmling_agent_providers.base import StreamingResponseProtocol, UsageLimits
 
-    AgentType = (
-        Literal["pydantic_ai", "human", "litellm"] | AgentProvider | Callable[..., Any]
-    )
+from llmling_agent_providers.base import (
+    AgentProvider,
+)
+
+
+AgentType = Literal["pydantic_ai", "human"] | AgentProvider | Callable[..., Any]
 
 logger = get_logger(__name__)
 
@@ -83,6 +85,8 @@ TDeps = TypeVar("TDeps", default=None)
 
 
 class AgentKwargs(TypedDict, total=False):
+    """Keyword arguments for configuring an Agent instance."""
+
     # Core Identity
     provider: AgentType
     description: str | None
@@ -100,7 +104,7 @@ class AgentKwargs(TypedDict, total=False):
 
     # Execution Settings
     retries: int
-    result_retries: int | None
+    output_retries: int | None
     end_strategy: EndStrategy
     defer_model_check: bool
 
@@ -113,7 +117,6 @@ class AgentKwargs(TypedDict, total=False):
     debug: bool
 
 
-@track_agent("Agent")
 class Agent[TDeps](MessageNode[TDeps, str], TaskManagerMixin):
     """Agent for AI-powered interaction with LLMling resources and tools.
 
@@ -136,7 +139,7 @@ class Agent[TDeps](MessageNode[TDeps, str], TaskManagerMixin):
         agent_name: AgentName
         previous_tools: dict[str, bool]
         new_tools: dict[str, bool]
-        timestamp: datetime = field(default_factory=datetime.now)
+        timestamp: datetime = field(default_factory=get_now)
 
     # this fixes weird mypy issue
     conversation: ConversationManager
@@ -146,7 +149,7 @@ class Agent[TDeps](MessageNode[TDeps, str], TaskManagerMixin):
     run_failed = Signal(str, Exception)
     agent_reset = Signal(AgentReset)
 
-    def __init__(
+    def __init__(  # noqa: PLR0915
         # we dont use AgentKwargs here so that we can work with explicit ones in the ctor
         self,
         name: str = "llmling-agent",
@@ -163,7 +166,7 @@ class Agent[TDeps](MessageNode[TDeps, str], TaskManagerMixin):
         mcp_servers: Sequence[str | MCPServerConfig] | None = None,
         resources: Sequence[Resource | PromptType | str] = (),
         retries: int = 1,
-        result_retries: int | None = None,
+        output_retries: int | None = None,
         end_strategy: EndStrategy = "early",
         defer_model_check: bool = False,
         input_provider: InputProvider | None = None,
@@ -183,7 +186,7 @@ class Agent[TDeps](MessageNode[TDeps, str], TaskManagerMixin):
                 - str/UUID: Session identifier
                 - SessionQuery: Query to recover conversation
                 - MemoryConfig: Complete memory configuration
-            model: The default model to use (defaults to GPT-4)
+            model: The default model to use (defaults to GPT-5)
             system_prompt: Static system prompts to use for this agent
             name: Name of the agent for logging
             description: Description of the Agent ("what it can do")
@@ -192,7 +195,7 @@ class Agent[TDeps](MessageNode[TDeps, str], TaskManagerMixin):
             mcp_servers: MCP servers to connect to
             resources: Additional resources to load
             retries: Default number of retries for failed operations
-            result_retries: Max retries for result validation (defaults to retries)
+            output_retries: Max retries for result validation (defaults to retries)
             end_strategy: Strategy for handling tool calls that are requested alongside
                           a final result
             defer_model_check: Whether to defer model evaluation until first run
@@ -229,6 +232,7 @@ class Agent[TDeps](MessageNode[TDeps, str], TaskManagerMixin):
             context=ctx,
             description=description,
             enable_logging=memory_cfg.enable,
+            mcp_servers=mcp_servers,
         )
         # Initialize runtime
         match runtime:
@@ -238,6 +242,9 @@ class Agent[TDeps](MessageNode[TDeps, str], TaskManagerMixin):
                 ctx.runtime = RuntimeConfig.from_config(runtime)
             case RuntimeConfig():
                 ctx.runtime = runtime
+            case _:
+                msg = f"Invalid runtime type: {type(runtime)}"
+                raise TypeError(msg)
 
         runtime_provider = RuntimePromptProvider(ctx.runtime)
         ctx.definition.prompt_manager.providers["runtime"] = runtime_provider
@@ -267,7 +274,7 @@ class Agent[TDeps](MessageNode[TDeps, str], TaskManagerMixin):
                     model=model,
                     retries=retries,
                     end_strategy=end_strategy,
-                    result_retries=result_retries,
+                    output_retries=output_retries,
                     defer_model_check=defer_model_check,
                     debug=debug,
                     context=ctx,
@@ -282,17 +289,6 @@ class Agent[TDeps](MessageNode[TDeps, str], TaskManagerMixin):
                 self._provider = CallbackProvider(
                     provider, name=name, debug=debug, context=ctx
                 )
-            case "litellm":
-                validate_import("litellm", "litellm")
-                from llmling_agent_providers.litellm_provider import LiteLLMProvider
-
-                self._provider = LiteLLMProvider(
-                    name=name,
-                    debug=debug,
-                    retries=retries,
-                    context=ctx,
-                    model=model,
-                )
             case AgentProvider():
                 self._provider = provider
                 self._provider.context = ctx
@@ -484,7 +480,8 @@ class Agent[TDeps](MessageNode[TDeps, str], TaskManagerMixin):
         """
         from llmling_agent_providers.callback import CallbackProvider
 
-        name = name or callback.__name__ or "processor"
+        name = name or getattr(callback, "__name__", "processor")
+        name = name or "processor"
         provider = CallbackProvider(callback, name=name)
         return Agent[None](provider=provider, name=name, debug=debug, **kwargs)
 
@@ -512,7 +509,7 @@ class Agent[TDeps](MessageNode[TDeps, str], TaskManagerMixin):
 
     def set_result_type(
         self,
-        result_type: type[TResult] | str | ResponseDefinition | None,
+        result_type: type[TResult] | str | StructuredResponseConfig | None,
         *,
         tool_name: str | None = None,
         tool_description: str | None = None,
@@ -559,11 +556,6 @@ class Agent[TDeps](MessageNode[TDeps, str], TaskManagerMixin):
                 from llmling_agent_providers.human import HumanProvider
 
                 self._provider = HumanProvider(name=name, debug=debug)
-            case "litellm":
-                validate_import("litellm", "litellm")
-                from llmling_agent_providers.litellm_provider import LiteLLMProvider
-
-                self._provider = LiteLLMProvider(model=model, name=name, debug=debug)
             case Callable():
                 from llmling_agent_providers.callback import CallbackProvider
 
@@ -575,7 +567,7 @@ class Agent[TDeps](MessageNode[TDeps, str], TaskManagerMixin):
         self._provider.model_changed.connect(self.model_changed)
         self._provider.tool_used.connect(self.tool_used)
         self._provider.model_changed.connect(self.model_changed)
-        self._provider.context = self._context
+        self._provider.context = self._context  # pyright: ignore[reportAttributeAccessIssue]
 
     @overload
     def to_structured(
@@ -589,7 +581,7 @@ class Agent[TDeps](MessageNode[TDeps, str], TaskManagerMixin):
     @overload
     def to_structured[TResult](
         self,
-        result_type: type[TResult] | str | ResponseDefinition,
+        result_type: type[TResult] | str | StructuredResponseConfig,
         *,
         tool_name: str | None = None,
         tool_description: str | None = None,
@@ -597,7 +589,7 @@ class Agent[TDeps](MessageNode[TDeps, str], TaskManagerMixin):
 
     def to_structured[TResult](
         self,
-        result_type: type[TResult] | str | ResponseDefinition | None,
+        result_type: type[TResult] | str | StructuredResponseConfig | None,
         *,
         tool_name: str | None = None,
         tool_description: str | None = None,
@@ -700,10 +692,11 @@ class Agent[TDeps](MessageNode[TDeps, str], TaskManagerMixin):
         result_type: type[TResult] | None = None,
         model: ModelType = None,
         store_history: bool = True,
-        tool_choice: bool | str | list[str] = True,
+        tool_choice: str | list[str] | None = None,
         usage_limits: UsageLimits | None = None,
         message_id: str | None = None,
         conversation_id: str | None = None,
+        messages: list[ChatMessage[Any]] | None = None,
         wait_for_connections: bool | None = None,
     ) -> ChatMessage[TResult]:
         """Run agent with prompt and get response.
@@ -714,15 +707,12 @@ class Agent[TDeps](MessageNode[TDeps, str], TaskManagerMixin):
             model: Optional model override
             store_history: Whether the message exchange should be added to the
                             context window
-            tool_choice: Control tool usage:
-                - True: Allow all tools
-                - False: No tools
-                - str: Use specific tool
-                - list[str]: Allow specific tools
+            tool_choice: Filter tool choice by name
             usage_limits: Optional usage limits for the model
             message_id: Optional message id for the returned message.
                         Automatically generated if not provided.
             conversation_id: Optional conversation id for the returned message.
+            messages: Optional list of messages to replace the conversation history
             wait_for_connections: Whether to wait for connected agents to complete
 
         Returns:
@@ -733,21 +723,14 @@ class Agent[TDeps](MessageNode[TDeps, str], TaskManagerMixin):
         """
         """Run agent with prompt and get response."""
         message_id = message_id or str(uuid4())
-
-        tools = await self.tools.get_tools(state="enabled")
-        match tool_choice:
-            case str():
-                tools = [t for t in tools if t.name == tool_choice]
-            case list():
-                tools = [t for t in tools if t.name in tool_choice]
-            case False:
-                tools = []
-            case True | None:
-                pass  # Keep all tools
+        tools = await self.tools.get_tools(state="enabled", names=tool_choice)
         self.set_result_type(result_type)
         start_time = time.perf_counter()
         sys_prompt = await self.sys_prompts.format_system_prompt(self)
-        message_history = self.conversation.get_history()
+
+        message_history = (
+            messages if messages is not None else self.conversation.get_history()
+        )
         try:
             result = await self._provider.generate_response(
                 *await convert_prompts(prompts),
@@ -788,11 +771,12 @@ class Agent[TDeps](MessageNode[TDeps, str], TaskManagerMixin):
         *prompt: AnyPromptType | PIL.Image.Image | os.PathLike[str],
         result_type: type[TResult] | None = None,
         model: ModelType = None,
-        tool_choice: bool | str | list[str] = True,
+        tool_choice: str | list[str] | None = None,
         store_history: bool = True,
         usage_limits: UsageLimits | None = None,
         message_id: str | None = None,
         conversation_id: str | None = None,
+        messages: list[ChatMessage[Any]] | None = None,
         wait_for_connections: bool | None = None,
     ) -> AsyncIterator[StreamingResponseProtocol[TResult]]:
         """Run agent with prompt and get a streaming response.
@@ -801,17 +785,14 @@ class Agent[TDeps](MessageNode[TDeps, str], TaskManagerMixin):
             prompt: User query or instruction
             result_type: Optional type for structured responses
             model: Optional model override
-            tool_choice: Control tool usage:
-                - True: Allow all tools
-                - False: No tools
-                - str: Use specific tool
-                - list[str]: Allow specific tools
+            tool_choice: Filter tool choice by name
             store_history: Whether the message exchange should be added to the
                            context window
             usage_limits: Optional usage limits for the model
             message_id: Optional message id for the returned message.
                         Automatically generated if not provided.
             conversation_id: Optional conversation id for the returned message.
+            messages: Optional list of messages to replace the conversation history
             wait_for_connections: Whether to wait for connected agents to complete
 
         Returns:
@@ -825,18 +806,11 @@ class Agent[TDeps](MessageNode[TDeps, str], TaskManagerMixin):
         self.set_result_type(result_type)
         start_time = time.perf_counter()
         sys_prompt = await self.sys_prompts.format_system_prompt(self)
-        tools = await self.tools.get_tools(state="enabled")
-        match tool_choice:
-            case str():
-                tools = [t for t in tools if t.name == tool_choice]
-            case list():
-                tools = [t for t in tools if t.name in tool_choice]
-            case False:
-                tools = []
-            case True | None:
-                pass  # Keep all tools
+        tools = await self.tools.get_tools(state="enabled", names=tool_choice)
+        message_history = (
+            messages if messages is not None else self.conversation.get_history()
+        )
         try:
-            message_history = self.conversation.get_history()
             async with self._provider.stream_response(
                 *prompts,
                 message_id=message_id,
@@ -853,19 +827,14 @@ class Agent[TDeps](MessageNode[TDeps, str], TaskManagerMixin):
                 cost_info = None
                 model_name = stream.model_name  # type: ignore
                 if model_name:
-                    cost_info = await TokenCost.from_usage(
-                        usage,
-                        model_name,
-                        str(user_msg.content),
-                        str(stream.formatted_content),  # type: ignore
-                    )
+                    cost_info = await TokenCost.from_usage(usage, model_name)
                 response_msg = ChatMessage[TResult](
                     content=cast(TResult, stream.formatted_content),  # type: ignore
                     role="assistant",
                     name=self.name,
                     model=model_name,
                     message_id=message_id,
-                    conversation_id=conversation_id,
+                    conversation_id=user_msg.conversation_id,
                     cost_info=cost_info,
                     response_time=time.perf_counter() - start_time,
                     # provider_extra=stream.provider_extra or {},
@@ -1003,6 +972,115 @@ class Agent[TDeps](MessageNode[TDeps, str], TaskManagerMixin):
             logger.exception(msg)
             raise JobError(msg) from e
 
+    @asynccontextmanager
+    @track_action("Calling Agent.iterate_run: {prompts}")
+    async def iterate_run(
+        self,
+        *prompts: AnyPromptType | PIL.Image.Image | os.PathLike[str],
+        message_id: str | None = None,
+        message_history: list[ChatMessage[Any]] | None = None,
+        tools: list[Tool] | None = None,
+        result_type: type[TResult] | None = None,
+        usage_limits: UsageLimits | None = None,
+        model: ModelType = None,
+        system_prompt: str | None = None,
+        tool_choice: str | list[str] | None = None,
+        conversation_id: str | None = None,
+        store_history: bool = True,
+    ) -> AsyncIterator[AgentRun[TDeps, TResult]]:
+        """Run the agent step-by-step, yielding an object to observe the execution graph.
+
+        Args:
+            *prompts: User query/instructions (text, images, paths, BasePrompts).
+            message_id: Optional unique ID for this run attempt. Generated if None.
+            message_history: Optional list of messages to replace current history.
+            tools: Optional sequence of tools to use instead of agent's default tools.
+            result_type: Optional type for structured responses.
+            usage_limits: Optional usage limits (provider support may vary).
+            model: Optional model override for this run.
+            system_prompt: Optional system prompt override for this run.
+            tool_choice: Filter agent's tools by name (ignored if `tools` is provided).
+            conversation_id: Optional ID to associate with the conversation context.
+            store_history: Whether to store the conversation in agent's history.
+
+        Yields:
+            An AgentRun object for iterating over execution nodes.
+
+        Example: (Same as before)
+            async with agent.iterate_run("Capital of France?") as agent_run:
+                async for node in agent_run:
+                    print(f"Processing: {type(node).__name__}")
+                print(f"Final result: {agent_run.result.output}")
+
+        Note: (Same as before regarding history management)
+        """
+        run_message_id = message_id or str(uuid4())
+        start_time = time.perf_counter()
+        logger.info("Starting agent iteration run_id=%s", run_message_id)
+        converted_prompts = await convert_prompts(prompts)
+        if not converted_prompts:
+            msg = "No prompts provided for iteration."
+            logger.error(msg)
+            raise ValueError(msg)
+
+        # Prepare user message for conversation history
+        user_msg = None
+        if store_history:
+            user_msg, _ = await self.pre_run(*prompts)
+
+        if tools is None:
+            effective_tools = await self.tools.get_tools(
+                state="enabled", names=tool_choice
+            )
+        else:
+            effective_tools = tools  # Use the direct override
+
+        self.set_result_type(result_type)
+        effective_system_prompt = (
+            system_prompt
+            if system_prompt is not None
+            else await self.sys_prompts.format_system_prompt(self)
+        )
+        effective_message_history = (
+            message_history
+            if message_history is not None
+            else self.conversation.get_history()
+        )
+        try:
+            async with self._provider.iterate_run(
+                *converted_prompts,
+                # Pass consistent arguments to the provider
+                message_id=run_message_id,
+                message_history=effective_message_history,
+                tools=effective_tools,
+                result_type=result_type,
+                usage_limits=usage_limits,
+                model=model,
+                system_prompt=effective_system_prompt,
+            ) as agent_run:
+                yield agent_run
+                # Store conversation history if requested
+                if store_history and user_msg and agent_run.result:
+                    response_msg = ChatMessage[TResult](
+                        content=agent_run.result.output,
+                        role="assistant",
+                        name=self.name,
+                        model=getattr(agent_run.result, "model_name", None),
+                        message_id=run_message_id,
+                        conversation_id=conversation_id or user_msg.conversation_id,
+                        response_time=time.perf_counter() - start_time,
+                    )
+                    self.conversation.add_chat_messages([user_msg, response_msg])
+                    msg = "Stored conversation history for run_id=%s"
+                    logger.debug(msg, run_message_id)
+
+            logger.info("Agent iteration run_id=%s completed.", run_message_id)
+
+        except Exception as e:
+            logger.exception("Agent iteration run_id=%s failed.", run_message_id)
+            self.run_failed.emit(f"Agent iteration failed: {e}", e)
+            raise
+
     async def run_in_background(
         self,
         *prompt: AnyPromptType | PIL.Image.Image | os.PathLike[str],
@@ -1150,7 +1228,7 @@ class Agent[TDeps](MessageNode[TDeps, str], TaskManagerMixin):
 
     def register_worker(
         self,
-        worker: AnyAgent[Any, Any],
+        worker: MessageNode[Any, Any],
         *,
         name: str | None = None,
         reset_history_on_run: bool = True,
@@ -1283,13 +1361,14 @@ class Agent[TDeps](MessageNode[TDeps, str], TaskManagerMixin):
 if __name__ == "__main__":
     # import logging
 
-    sys_prompt = "Open browser with google, please"
-    _model = "openai:gpt-4o-mini"
+    sys_prompt = "Open browser with google,"
+    _model = "openai:gpt-5-mini"
 
     async def main():
         async with Agent[None](model=_model, tools=["webbrowser.open"]) as agent:
-            print(agent.tools.list_items())
-            result = await agent.run(sys_prompt)
-            print(result.data)
+            agent.tool_used.connect(print)
+            async with agent.iterate_run(sys_prompt) as stream:
+                async for chunk in stream:
+                    print(chunk)
 
     asyncio.run(main())
