@@ -14,10 +14,11 @@ from llmling_agent.prompts.conversion_manager import ConversionManager
 
 if TYPE_CHECKING:
     from llmling import RuntimeConfig
+    from mcp import types
 
+    from llmling_agent import AgentPool
     from llmling_agent.agent import AnyAgent
     from llmling_agent.config.capabilities import Capabilities
-    from llmling_agent.delegation.pool import AgentPool
     from llmling_agent.models.agents import AgentConfig
     from llmling_agent.tools.base import Tool
     from llmling_agent_input.base import InputProvider
@@ -97,6 +98,12 @@ class AgentContext[TDeps](NodeContext[TDeps]):
         assert self.node_name, "No agent name available"
         return self.pool.agents[self.node_name]
 
+    @property
+    def process_manager(self):
+        """Get process manager from pool."""
+        assert self.pool, "No agent pool available"
+        return self.pool.process_manager
+
     async def handle_confirmation(
         self,
         tool: Tool,
@@ -114,3 +121,12 @@ class AgentContext[TDeps](NodeContext[TDeps]):
             return "allow"
         history = self.agent.conversation.get_history() if self.pool else []
         return await provider.get_tool_confirmation(self, tool, args, history)
+
+    async def handle_elicitation(
+        self,
+        params: types.ElicitRequestParams,
+    ) -> types.ElicitResult | types.ErrorData:
+        """Handle elicitation request for additional information."""
+        provider = self.get_input_provider()
+        history = self.agent.conversation.get_history() if self.pool else []
+        return await provider.get_elicitation(self, params, history)
