@@ -2,27 +2,25 @@
 
 from __future__ import annotations
 
-from collections.abc import AsyncIterator, Callable, Mapping
+from collections.abc import Callable, Mapping
 from typing import TYPE_CHECKING, Any, Literal, cast, overload
 
-from pydantic import BaseModel
+from schemez import Schema
 from typing_extensions import TypeVar
 
 from llmling_agent.log import get_logger
 from llmling_agent.messaging.messages import ChatMessage
 from llmling_agent.tools.base import Tool
-from llmling_agent.utils.basemodel_convert import get_ctor_basemodel
 
 
 if TYPE_CHECKING:
-    from collections.abc import Sequence
+    from collections.abc import AsyncIterator, Sequence
 
     from toprompt import AnyPromptType
 
+    from llmling_agent import AgentPool, MessageNode
     from llmling_agent.agent import AnyAgent
     from llmling_agent.delegation.base_team import BaseTeam
-    from llmling_agent.delegation.pool import AgentPool
-    from llmling_agent.messaging.messagenode import MessageNode
 
 TResult = TypeVar("TResult", default=str)
 TDeps = TypeVar("TDeps", default=None)
@@ -33,28 +31,28 @@ type EndCondition = Callable[[list[ChatMessage[Any]], ChatMessage[Any]], bool]
 logger = get_logger(__name__)
 
 
-class LLMPick(BaseModel):
+class LLMPick(Schema):
     """Decision format for LLM response."""
 
     selection: str  # The label/name of the selected option
     reason: str
 
 
-class Pick[T](BaseModel):
+class Pick[T](Schema):
     """Type-safe decision with original object."""
 
     selection: T
     reason: str
 
 
-class LLMMultiPick(BaseModel):
+class LLMMultiPick(Schema):
     """Multiple selection format for LLM response."""
 
     selections: list[str]  # Labels of selected options
     reason: str
 
 
-class MultiPick[T](BaseModel):
+class MultiPick[T](Schema):
     """Type-safe multiple selection with original objects."""
 
     selections: list[T]
@@ -75,7 +73,7 @@ def get_label(item: Any) -> str:
         - types use __name__
         - others use __repr__ for unique identifiable string
     """
-    from llmling_agent.messaging.messagenode import MessageNode
+    from llmling_agent import MessageNode
 
     match item:
         case str():
@@ -207,8 +205,8 @@ class Interactions[TDeps, TResult]:
         # Get items and create label mapping
         from toprompt import to_prompt
 
+        from llmling_agent import AgentPool
         from llmling_agent.delegation.base_team import BaseTeam
-        from llmling_agent.delegation.pool import AgentPool
 
         match selections:
             case dict():
@@ -318,8 +316,8 @@ Select ONE option by its exact label."""
         """
         from toprompt import to_prompt
 
+        from llmling_agent import AgentPool
         from llmling_agent.delegation.base_team import BaseTeam
-        from llmling_agent.delegation.pool import AgentPool
 
         match selections:
             case Mapping():
@@ -408,7 +406,7 @@ List your selections, one per line, followed by your reasoning."""
         from py2openai import create_constructor_schema
 
         # Create model for single instance
-        item_model = get_ctor_basemodel(as_type)
+        item_model = Schema.for_class_ctor(as_type)
 
         # Create extraction prompt
         final_prompt = prompt or f"Extract {as_type.__name__} from: {text}"
@@ -417,7 +415,7 @@ List your selections, one per line, followed by your reasoning."""
 
         if mode == "structured":
 
-            class Extraction(BaseModel):
+            class Extraction(Schema):
                 instance: item_model  # type: ignore
                 # explanation: str | None = None
 
@@ -469,7 +467,7 @@ List your selections, one per line, followed by your reasoning."""
         """
         from py2openai import create_constructor_schema
 
-        item_model = get_ctor_basemodel(as_type)
+        item_model = Schema.for_class_ctor(as_type)
 
         instances: list[T] = []
         schema_obj = create_constructor_schema(as_type)
@@ -484,7 +482,7 @@ List your selections, one per line, followed by your reasoning."""
         if mode == "structured":
             # Create model for individual instance
 
-            class Extraction(BaseModel):
+            class Extraction(Schema):
                 instances: list[item_model]  # type: ignore
                 # explanation: str | None = None
 
