@@ -31,9 +31,6 @@ class Capabilities(EventedModel):
     history_access: AccessLevel = "none"
     """Level of access to conversation history."""
 
-    stats_access: AccessLevel = "none"
-    """Level of access to usage statistics."""
-
     # Resource capabilities
     can_load_resources: bool = False
     """Whether the agent can load and access resource content."""
@@ -41,6 +38,12 @@ class Capabilities(EventedModel):
     can_list_resources: bool = False
     """Whether the agent can discover available resources."""
 
+    can_read_files: bool = False
+    """Whether the agent can read local and remote files."""
+
+    can_list_directories: bool = False
+    """Whether the agent can list directories and their contents."""
+
     # Tool management
     can_register_tools: bool = False
     """Whether the agent can register importable functions as tools."""
@@ -62,6 +65,9 @@ class Capabilities(EventedModel):
     can_execute_commands: bool = False
     """Whether the agent can execute CLI commands (use at your own risk)."""
 
+    can_manage_processes: bool = False
+    """Whether the agent can start and manage background processes."""
+
     # Agent creation
     can_create_workers: bool = False
     """Whether the agent can create worker agents (as tools)."""
@@ -89,21 +95,19 @@ class Capabilities(EventedModel):
         Example:
             required in agent.capabilities  # Can agent fulfill requirements?
         """
-        # Check all boolean capabilities
-        for field in self.__fields__:
-            if isinstance(getattr(required, field), bool):  # noqa: SIM102
-                if getattr(required, field) and not getattr(self, field):
-                    return False
+        # Check all boolean capabilities using any() to find any missing required flag
+        if any(
+            getattr(required, field) and not getattr(self, field)
+            for field in type(self).model_fields
+            if isinstance(getattr(required, field), bool)
+        ):
+            return False
 
         # Check access levels (none < own < all)
         access_order = {"none": 0, "own": 1, "all": 2}
-        for field in ("history_access", "stats_access"):
-            required_level = access_order[getattr(required, field)]
-            self_level = access_order[getattr(self, field)]
-            if required_level > self_level:
-                return False
-
-        return True
+        required_level = access_order[required.history_access]
+        self_level = access_order[self.history_access]
+        return not required_level > self_level
 
     def has_capability(self, capability: str) -> bool:
         """Check if a specific capability is enabled.
