@@ -4,50 +4,53 @@ from __future__ import annotations
 
 import asyncio
 from contextlib import (
-    AbstractAsyncContextManager,
     AsyncExitStack,
     asynccontextmanager,
     suppress,
 )
-import signal
+import os
 from typing import TYPE_CHECKING, Any, Self, Unpack, cast, overload
 
 from llmling import BaseRegistry, LLMLingError
 from typing_extensions import TypeVar
 
-from llmling_agent.agent import Agent, AnyAgent, StructuredAgent
-from llmling_agent.common_types import AgentName, NodeName
+from llmling_agent.agent import Agent, StructuredAgent
+from llmling_agent.common_types import NodeName
 from llmling_agent.delegation.message_flow_tracker import MessageFlowTracker
 from llmling_agent.delegation.team import Team
 from llmling_agent.delegation.teamrun import TeamRun
 from llmling_agent.log import get_logger
-from llmling_agent.mcp_server.manager import MCPManager
 from llmling_agent.messaging.messageemitter import MessageEmitter
-from llmling_agent.models.forward_targets import (
+from llmling_agent.talk import Talk, TeamTalk
+from llmling_agent.talk.registry import ConnectionRegistry
+from llmling_agent.tasks import TaskRegistry
+from llmling_agent_config.forward_targets import (
     CallableConnectionConfig,
     FileConnectionConfig,
     NodeConnectionConfig,
 )
-from llmling_agent.talk import Talk, TeamTalk
-from llmling_agent.talk.registry import ConnectionRegistry
-from llmling_agent.tasks import TaskRegistry
+from llmling_agent_config.workers import AgentWorkerConfig, TeamWorkerConfig
 
 
 if TYPE_CHECKING:
     from collections.abc import AsyncIterator, Sequence
+    from contextlib import (
+        AbstractAsyncContextManager,
+    )
     from types import TracebackType
 
     from psygnal.containers._evented_dict import DictEvents
 
+    from llmling_agent.agent import AnyAgent
     from llmling_agent.agent.agent import AgentKwargs
-    from llmling_agent.common_types import SessionIdType, StrPath
+    from llmling_agent.common_types import AgentName, SessionIdType, StrPath
     from llmling_agent.delegation.base_team import BaseTeam
     from llmling_agent.messaging.eventnode import EventNode
     from llmling_agent.messaging.messagenode import MessageNode
     from llmling_agent.models.manifest import AgentsManifest
-    from llmling_agent.models.result_types import ResponseDefinition
-    from llmling_agent.models.session import SessionQuery
-    from llmling_agent.models.task import Job
+    from llmling_agent_config.result_types import StructuredResponseConfig
+    from llmling_agent_config.session import SessionQuery
+    from llmling_agent_config.task import Job
     from llmling_agent_input.base import InputProvider
 
 
@@ -96,13 +99,14 @@ class AgentPool[TPoolDeps](BaseRegistry[NodeName, MessageEmitter[Any, Any]]):
             RuntimeError: If node initialization fails
         """
         super().__init__()
+        from llmling_agent.mcp_server.manager import MCPManager
         from llmling_agent.models.manifest import AgentsManifest
         from llmling_agent.storage import StorageManager
 
         match manifest:
             case None:
                 self.manifest = AgentsManifest()
-            case str():
+            case str() | os.PathLike():
                 self.manifest = AgentsManifest.from_file(manifest)
             case AgentsManifest():
                 self.manifest = manifest
@@ -122,6 +126,11 @@ class AgentPool[TPoolDeps](BaseRegistry[NodeName, MessageEmitter[Any, Any]]):
         # Register tasks from manifest
         for name, task in self.manifest.jobs.items():
             self._tasks.register(name, task)
+
+        # Initialize process manager for background processes
+        from llmling_agent.agent.process_manager import ProcessManager
+
+        self.process_manager = ProcessManager()
         self.pool_talk = TeamTalk[Any].from_nodes(list(self.nodes.values()))
         if self.manifest.pool_server and self.manifest.pool_server.enabled:
             from llmling_agent.resource_providers.pool import PoolResourceProvider
@@ -200,6 +209,8 @@ class AgentPool[TPoolDeps](BaseRegistry[NodeName, MessageEmitter[Any, Any]]):
 
     async def cleanup(self):
         """Clean up all agents."""
+        # Clean up background processes first
+        await self.process_manager.cleanup()
         await self.exit_stack.aclose()
         self.clear()
 
@@ -382,26 +393,13 @@ class AgentPool[TPoolDeps](BaseRegistry[NodeName, MessageEmitter[Any, Any]]):
 
     async def run_event_loop(self):
         """Run pool in event-watching mode until interrupted."""
-        import sys
-
         print("Starting event watch mode...")
         print("Active nodes: ", ", ".join(self.list_nodes()))
         print("Press Ctrl+C to stop")
 
-        stop_event = asyncio.Event()
-
-        if sys.platform != "win32":
-            # Unix: Use signal handlers
-            loop = asyncio.get_running_loop()
-            for sig in (signal.SIGINT, signal.SIGTERM):
-                loop.add_signal_handler(sig, stop_event.set)
+        with suppress(KeyboardInterrupt):
             while True:
                 await asyncio.sleep(1)
-        else:
-            # Windows: Use keyboard interrupt
-            with suppress(KeyboardInterrupt):
-                while True:
-                    await asyncio.sleep(1)
 
     @property
     def agents(self) -> dict[str, AnyAgent[Any, Any]]:
@@ -422,7 +420,7 @@ class AgentPool[TPoolDeps](BaseRegistry[NodeName, MessageEmitter[Any, Any]]):
     @property
     def nodes(self) -> dict[str, MessageNode[Any, Any]]:
         """Get agents dict (backward compatibility)."""
-        from llmling_agent.messaging.messagenode import MessageNode
+        from llmling_agent import MessageNode
 
         return {i.name: i for i in self._items.values() if isinstance(i, MessageNode)}
 
@@ -613,7 +611,7 @@ class AgentPool[TPoolDeps](BaseRegistry[NodeName, MessageEmitter[Any, Any]]):
         # Register in pool
         agent_name = new_agent.name
         self.manifest.agents[agent_name] = new_config
-        self.agents[agent_name] = new_agent
+        self.register(agent_name, new_agent)
         return await self.exit_stack.enter_async_context(new_agent)
 
     @overload
@@ -712,14 +710,18 @@ class AgentPool[TPoolDeps](BaseRegistry[NodeName, MessageEmitter[Any, Any]]):
         """Set up workers for an agent from configuration."""
         for worker_config in agent.context.config.workers:
             try:
-                worker = self.get_agent(worker_config.name)
-                agent.register_worker(
-                    worker,
-                    name=worker_config.name,
-                    reset_history_on_run=worker_config.reset_history_on_run,
-                    pass_message_history=worker_config.pass_message_history,
-                    share_context=worker_config.share_context,
-                )
+                worker = self.nodes[worker_config.name]
+                match worker_config:
+                    case TeamWorkerConfig():
+                        agent.register_worker(worker)
+                    case AgentWorkerConfig():
+                        agent.register_worker(
+                            worker,
+                            name=worker_config.name,
+                            reset_history_on_run=worker_config.reset_history_on_run,
+                            pass_message_history=worker_config.pass_message_history,
+                            share_context=worker_config.share_context,
+                        )
             except KeyError as e:
                 msg = f"Worker agent {worker_config.name!r} not found"
                 raise ValueError(msg) from e
@@ -848,7 +850,7 @@ class AgentPool[TPoolDeps](BaseRegistry[NodeName, MessageEmitter[Any, Any]]):
         self,
         name: AgentName,
         *,
-        result_type: type[TResult] | str | ResponseDefinition,
+        result_type: type[TResult] | str | StructuredResponseConfig,
         **kwargs: Unpack[AgentKwargs],
     ) -> StructuredAgent[Any, TResult]: ...
 
@@ -856,7 +858,7 @@ class AgentPool[TPoolDeps](BaseRegistry[NodeName, MessageEmitter[Any, Any]]):
         self,
         name: AgentName,
         *,
-        result_type: type[Any] | str | ResponseDefinition | None = None,
+        result_type: type[Any] | str | StructuredResponseConfig | None = None,
         **kwargs: Unpack[AgentKwargs],
     ) -> Agent[Any] | StructuredAgent[Any, Any]:
         """Add a new permanent agent to the pool.
@@ -867,7 +869,7 @@ class AgentPool[TPoolDeps](BaseRegistry[NodeName, MessageEmitter[Any, Any]]):
                 - None: Regular unstructured agent
                 - type: Python type for validation
                 - str: Name of response definition
-                - ResponseDefinition: Complete response definition
+                - StructuredResponseConfig: Complete response definition
             **kwargs: Additional agent configuration
 
         Returns:
@@ -931,7 +933,7 @@ if __name__ == "__main__":
     async def main():
         path = "src/llmling_agent/config_resources/agents.yml"
         async with AgentPool[None](path) as pool:
-            agent: Agent[Any] = pool.get_agent("overseer")
+            agent: Agent[None] = pool.get_agent("overseer")
             print(agent)
 
     import asyncio
