@@ -1,11 +1,10 @@
-"""Team execution management and monitoring."""
+"""Sequential, ordered group of agents / nodes."""
 
 from __future__ import annotations
 
 import asyncio
 from contextlib import AsyncExitStack, asynccontextmanager
 from dataclasses import dataclass, field
-from datetime import datetime
 from itertools import pairwise
 from time import perf_counter
 from typing import TYPE_CHECKING, Any, Literal
@@ -15,18 +14,20 @@ from llmling_agent.delegation.base_team import BaseTeam
 from llmling_agent.log import get_logger
 from llmling_agent.messaging.messages import AgentResponse, ChatMessage, TeamResponse
 from llmling_agent.talk.talk import Talk, TeamTalk
+from llmling_agent.utils.now import get_now
 
 
 if TYPE_CHECKING:
-    from collections.abc import AsyncIterator, Sequence
+    from collections.abc import AsyncGenerator, AsyncIterator, Sequence
+    from datetime import datetime
     import os
 
     import PIL.Image
     from tokonomics.pydanticai_cost import Usage
     from toprompt import AnyPromptType
 
+    from llmling_agent import MessageNode
     from llmling_agent.agent import AnyAgent
-    from llmling_agent.messaging.messagenode import MessageNode
     from llmling_agent_providers.base import StreamingResponseProtocol
 
 
@@ -48,7 +49,7 @@ class ExtendedTeamTalk(TeamTalk):
 
     def add_error(self, agent: str, error: str):
         """Track errors from AgentResponses."""
-        self.errors.append((agent, error, datetime.now()))
+        self.errors.append((agent, error, get_now()))
 
     @property
     def error_log(self) -> list[tuple[str, str, datetime]]:
@@ -139,7 +140,7 @@ class TeamRun[TDeps, TResult](BaseTeam[TDeps, TResult]):
     ) -> TeamResponse[TResult]:
         """Start execution with optional monitoring."""
         self._team_talk.clear()
-        start_time = datetime.now()
+        start_time = get_now()
         final_prompt = list(prompts)
         if self.shared_prompt:
             final_prompt.insert(0, self.shared_prompt)
@@ -247,7 +248,7 @@ class TeamRun[TDeps, TResult](BaseTeam[TDeps, TResult]):
                     )
                     streams.append(stream)  # type: ignore
                     # Wait for complete response for next agent
-                    async for chunk in stream.stream():
+                    async for chunk in stream.stream_output():
                         current_message = chunk
                         if stream.is_complete:
                             current_message = (stream.formatted_content,)  # type: ignore
@@ -269,16 +270,31 @@ class TeamRun[TDeps, TResult](BaseTeam[TDeps, TResult]):
                 def usage(self) -> Usage:
                     @dataclass
                     class Usage:
-                        total_tokens: int | None
-                        request_tokens: int | None
-                        response_tokens: int | None
+                        input_tokens: int
+                        output_tokens: int
 
-                    return Usage(0, 0, 0)
+                        @property
+                        def total_tokens(self) -> int:
+                            """Total number of tokens used."""
+                            return self.input_tokens + self.output_tokens
 
-                async def stream(self) -> AsyncIterator[str]:  # type: ignore
+                    return Usage(0, 0)
+
+                async def stream_output(self) -> AsyncGenerator[str, None]:  # type: ignore
                     for idx, stream in enumerate(self.streams):
                         self.current_stream_idx = idx
-                        async for chunk in stream.stream():
+                        async for chunk in stream.stream_output():
+                            yield chunk
+                            if idx == len(self.streams) - 1 and stream.is_complete:
+                                self.is_complete = True
+
+                async def stream_text(
+                    self,
+                    delta: bool = False,
+                ) -> AsyncGenerator[str, None]:
+                    for idx, stream in enumerate(self.streams):
+                        self.current_stream_idx = idx
+                        async for chunk in stream.stream_text(delta=delta):
                             yield chunk
                             if idx == len(self.streams) - 1 and stream.is_complete:
                                 self.is_complete = True
@@ -310,12 +326,12 @@ if __name__ == "__main__":
             agent1 = await pool.add_agent(
                 "analyzer",
                 system_prompt="You analyze text and find key points.",
-                model="openai:gpt-4o-mini",
+                model="openai:gpt-5-mini",
             )
             agent2 = await pool.add_agent(
                 "summarizer",
                 system_prompt="You create concise summaries.",
-                model="openai:gpt-4o-mini",
+                model="openai:gpt-5-mini",
             )
 
             # Create team and get monitored execution
