@@ -16,25 +16,27 @@ from llmling import (
     StaticPrompt,
 )
 from llmling.utils.importing import import_callable, import_class
-from pydantic import BaseModel, ConfigDict, Field, model_validator
+from pydantic import Field, model_validator
+from schemez import InlineSchemaDef
 from toprompt import render_prompt
 
 from llmling_agent.common_types import EndStrategy  # noqa: TC001
 from llmling_agent.config import Capabilities
-from llmling_agent.models.environment import (
-    AgentEnvironment,
+from llmling_agent.resource_providers.static import StaticResourceProvider
+from llmling_agent_config.environment import (
+    AgentEnvironment,  # noqa: TC001
     FileEnvironment,
     InlineEnvironment,
 )
-from llmling_agent.models.knowledge import Knowledge  # noqa: TC001
-from llmling_agent.models.nodes import NodeConfig
-from llmling_agent.models.providers import ProviderConfig  # noqa: TC001
-from llmling_agent.models.result_types import InlineResponseDefinition, ResponseDefinition
-from llmling_agent.models.session import MemoryConfig, SessionQuery
-from llmling_agent.models.tools import BaseToolConfig, ToolConfig
-from llmling_agent.models.toolsets import ToolsetConfig  # noqa: TC001
-from llmling_agent.resource_providers.static import StaticResourceProvider
-from llmling_agent_models import AnyModelConfig  # noqa: TC001
+from llmling_agent_config.knowledge import Knowledge  # noqa: TC001
+from llmling_agent_config.models import AnyModelConfig  # noqa: TC001
+from llmling_agent_config.nodes import NodeConfig
+from llmling_agent_config.providers import ProviderConfig  # noqa: TC001
+from llmling_agent_config.result_types import StructuredResponseConfig  # noqa: TC001
+from llmling_agent_config.session import MemoryConfig, SessionQuery
+from llmling_agent_config.tools import BaseToolConfig, ToolConfig  # noqa: TC001
+from llmling_agent_config.toolsets import ToolsetConfig  # noqa: TC001
+from llmling_agent_config.workers import WorkerConfig  # noqa: TC001
 
 
 if TYPE_CHECKING:
@@ -44,43 +46,11 @@ if TYPE_CHECKING:
 
 
 ToolConfirmationMode = Literal["always", "never", "per_tool"]
+ProviderName = Literal["pydantic_ai", "human"]
 
 logger = logging.getLogger(__name__)
 
 
-class WorkerConfig(BaseModel):
-    """Configuration for a worker agent.
-
-    Worker agents are agents that are registered as tools with a parent agent.
-    This allows building hierarchies and specializations of agents.
-    """
-
-    name: str
-    """Name of the agent to use as a worker"""
-
-    reset_history_on_run: bool = True
-    """Whether to clear worker's conversation history before each run.
-    True (default): Fresh conversation each time
-    False: Maintain conversation context between runs"""
-
-    pass_message_history: bool = False
-    """Whether to pass parent agent's message history to worker.
-    True: Worker sees parent's conversation context
-    False (default): Worker only sees current request"""
-
-    share_context: bool = False
-    """Whether to share parent agent's context/dependencies with worker.
-    True: Worker has access to parent's context data
-    False (default): Worker uses own isolated context"""
-
-    model_config = ConfigDict(frozen=True, use_attribute_docstrings=True, extra="forbid")
-
-    @classmethod
-    def from_str(cls, name: str) -> WorkerConfig:
-        """Create config from simple string form."""
-        return cls(name=name)
-
-
 class AgentConfig(NodeConfig):
     """Configuration for a single agent in the system.
 
@@ -95,7 +65,7 @@ class AgentConfig(NodeConfig):
     The configuration can be loaded from YAML or created programmatically.
     """
 
-    provider: ProviderConfig | Literal["pydantic_ai", "human", "litellm"] = "pydantic_ai"
+    provider: ProviderConfig | ProviderName = "pydantic_ai"
     """Provider configuration or shorthand type"""
 
     inherits: str | None = None
@@ -103,7 +73,7 @@ class AgentConfig(NodeConfig):
 
     model: str | AnyModelConfig | None = None
     """The model to use for this agent. Can be either a simple model name
-    string (e.g. 'openai:gpt-4') or a structured model definition."""
+    string (e.g. 'openai:gpt-5') or a structured model definition."""
 
     tools: list[ToolConfig | str] = Field(default_factory=list)
     """A list of tools to register with this agent."""
@@ -120,7 +90,7 @@ class AgentConfig(NodeConfig):
     session: str | SessionQuery | MemoryConfig | None = None
     """Session configuration for conversation recovery."""
 
-    result_type: str | ResponseDefinition | None = None
+    result_type: str | StructuredResponseConfig | None = None
     """Name of the response definition to use"""
 
     retries: int = 1
@@ -132,7 +102,7 @@ class AgentConfig(NodeConfig):
     result_tool_description: str | None = None
     """Custom description for the result tool"""
 
-    result_retries: int | None = None
+    output_retries: int | None = None
     """Max retries for result validation"""
 
     end_strategy: EndStrategy = "early"
@@ -176,21 +146,6 @@ class AgentConfig(NodeConfig):
         """Check if this config defines a structured agent."""
         return self.result_type is not None
 
-    @model_validator(mode="before")
-    @classmethod
-    def normalize_workers(cls, data: dict[str, Any]) -> dict[str, Any]:
-        """Convert string workers to WorkerConfig."""
-        if workers := data.get("workers"):
-            data["workers"] = [
-                WorkerConfig.from_str(w)
-                if isinstance(w, str)
-                else w
-                if isinstance(w, WorkerConfig)  # Keep existing WorkerConfig
-                else WorkerConfig(**w)  # Convert dict to WorkerConfig
-                for w in workers
-            ]
-        return data
-
     @model_validator(mode="before")
     @classmethod
     def validate_result_type(cls, data: dict[str, Any]) -> dict[str, Any]:
@@ -200,12 +155,12 @@ class AgentConfig(NodeConfig):
             # Extract response-specific settings
             tool_name = result_type.pop("result_tool_name", None)
             tool_description = result_type.pop("result_tool_description", None)
-            retries = result_type.pop("result_retries", None)
+            retries = result_type.pop("output_retries", None)
 
             # Convert remaining dict to ResponseDefinition
-            if "type" not in result_type:
-                result_type["type"] = "inline"
-            data["result_type"] = InlineResponseDefinition(**result_type)
+            if "type" not in result_type["response_schema"]:
+                result_type["response_schema"]["type"] = "inline"
+            data["result_type"]["response_schema"] = InlineSchemaDef(**result_type)
 
             # Apply extracted settings to agent config
             if tool_name:
@@ -213,7 +168,7 @@ class AgentConfig(NodeConfig):
             if tool_description:
                 data["result_tool_description"] = tool_description
             if retries is not None:
-                data["result_retries"] = retries
+                data["output_retries"] = retries
 
         return data
 
@@ -285,6 +240,9 @@ class AgentConfig(NodeConfig):
                 return self.session
             case None:
                 return MemoryConfig()
+            case _:
+                msg = f"Invalid session configuration: {self.session}"
+                raise ValueError(msg)
 
     def get_system_prompts(self) -> list[BasePrompt]:
         """Get all system prompts as BasePrompts."""
@@ -311,10 +269,9 @@ class AgentConfig(NodeConfig):
         - Shorthand type: Create default provider config
         """
         # If string shorthand is used, convert to default provider config
-        from llmling_agent.models.providers import (
+        from llmling_agent_config.providers import (
             CallbackProviderConfig,
             HumanProviderConfig,
-            LiteLLMProviderConfig,
             PydanticAIProviderConfig,
         )
 
@@ -325,14 +282,10 @@ class AgentConfig(NodeConfig):
                     provider_config = PydanticAIProviderConfig(model=self.model)
                 case "human":
                     provider_config = HumanProviderConfig()
-                case "litellm":
-                    provider_config = LiteLLMProviderConfig(
-                        model=self.model if isinstance(self.model, str) else None
-                    )
                 case _:
                     try:
                         fn = import_callable(provider_config)
-                        provider_config = CallbackProviderConfig(fn=fn)
+                        provider_config = CallbackProviderConfig(callback=fn)
                     except Exception:  # noqa: BLE001
                         msg = f"Invalid provider type: {provider_config}"
                         raise ValueError(msg)  # noqa: B904
@@ -400,7 +353,7 @@ class AgentConfig(NodeConfig):
 
 
 if __name__ == "__main__":
-    model = "openai:gpt-4o-mini"
+    model = "openai:gpt-5-mini"
     agent_cfg = AgentConfig(
         name="test_agent", model=model, tools=["crewai_tools.BraveSearchTool"]
     )  # type: ignore
