@@ -6,29 +6,39 @@ from functools import cached_property
 import logging
 from typing import TYPE_CHECKING, Any, Self
 
-from llmling import ConfigModel
 from pydantic import ConfigDict, Field, model_validator
+from schemez.schema import Schema
 
 from llmling_agent.models.agents import AgentConfig
-from llmling_agent.models.converters import ConversionConfig
-from llmling_agent.models.mcp_server import (
-    MCPServerBase,
-    MCPServerConfig,
+from llmling_agent.resource_registry import ResourceRegistry
+from llmling_agent_config.converters import ConversionConfig
+from llmling_agent_config.mcp_server import (
+    BaseMCPServerConfig,
+    MCPServerConfig,  # noqa: TC001
     PoolServerConfig,
-    StdioMCPServer,
+    StdioMCPServerConfig,
+)
+from llmling_agent_config.observability import ObservabilityConfig
+from llmling_agent_config.prompts import PromptConfig
+from llmling_agent_config.providers import BaseProviderConfig
+from llmling_agent_config.resources import (  # noqa: TC001
+    ResourceConfig,
+    SourceResourceConfig,
+)
+from llmling_agent_config.result_types import StructuredResponseConfig  # noqa: TC001
+from llmling_agent_config.storage import StorageConfig
+from llmling_agent_config.task import Job  # noqa: TC001
+from llmling_agent_config.teams import TeamConfig  # noqa: TC001
+from llmling_agent_config.ui import StdlibUIConfig, UIConfig  # noqa: TC001
+from llmling_agent_config.workers import (
+    AgentWorkerConfig,
+    BaseWorkerConfig,
+    TeamWorkerConfig,
 )
-from llmling_agent.models.observability import ObservabilityConfig
-from llmling_agent.models.prompts import PromptConfig
-from llmling_agent.models.providers import BaseProviderConfig
-from llmling_agent.models.result_types import ResponseDefinition  # noqa: TC001
-from llmling_agent.models.storage import StorageConfig
-from llmling_agent.models.task import Job  # noqa: TC001
-from llmling_agent.models.teams import TeamConfig  # noqa: TC001
 
 
 if TYPE_CHECKING:
-    from llmling_agent import AgentPool
-    from llmling_agent.agent import AnyAgent
+    from llmling_agent import AgentPool, AnyAgent
     from llmling_agent.common_types import StrPath
     from llmling_agent.prompts.manager import PromptManager
 
@@ -37,7 +47,7 @@ logger = logging.getLogger(__name__)
 
 
 # TODO: python 3.13: set defaults here
-class AgentsManifest(ConfigModel):
+class AgentsManifest(Schema):
     """Complete agent configuration manifest defining all available agents.
 
     This is the root configuration that:
@@ -53,6 +63,21 @@ class AgentsManifest(ConfigModel):
     INHERIT: str | list[str] | None = None
     """Inheritance references."""
 
+    resources: dict[str, ResourceConfig | str] = Field(default_factory=dict)
+    """Resource configurations defining available filesystems.
+
+    Supports both full config and URI shorthand:
+        resources:
+          docs: "file://./docs"  # shorthand
+          data:  # full config
+            type: "source"
+            uri: "s3://bucket/data"
+            cached: true
+    """
+
+    ui: UIConfig = Field(default_factory=StdlibUIConfig)
+    """UI configuration."""
+
     agents: dict[str, AgentConfig] = Field(default_factory=dict)
     """Mapping of agent IDs to their configurations"""
 
@@ -68,7 +93,7 @@ class AgentsManifest(ConfigModel):
     conversion: ConversionConfig = Field(default_factory=ConversionConfig)
     """Document conversion configuration."""
 
-    responses: dict[str, ResponseDefinition] = Field(default_factory=dict)
+    responses: dict[str, StructuredResponseConfig] = Field(default_factory=dict)
     """Mapping of response names to their definitions"""
 
     jobs: dict[str, Job] = Field(default_factory=dict)
@@ -89,6 +114,86 @@ class AgentsManifest(ConfigModel):
 
     model_config = ConfigDict(use_attribute_docstrings=True, extra="forbid")
 
+    @model_validator(mode="before")
+    @classmethod
+    def normalize_workers(cls, data: dict[str, Any]) -> dict[str, Any]:
+        """Convert string workers to appropriate WorkerConfig for all agents."""
+        teams = data.get("teams", {})
+        agents = data.get("agents", {})
+
+        # Process workers for all agents that have them
+        for agent_name, agent_config in agents.items():
+            if isinstance(agent_config, dict):
+                workers = agent_config.get("workers", [])
+            else:
+                workers = agent_config.workers
+
+            if workers:
+                normalized: list[BaseWorkerConfig] = []
+
+                for worker in workers:
+                    match worker:
+                        case str() as name:
+                            # Determine type based on presence in teams/agents
+                            if name in teams:
+                                normalized.append(TeamWorkerConfig(name=name))
+                            elif name in agents:
+                                normalized.append(AgentWorkerConfig(name=name))
+                            else:
+                                # Default to agent if type can't be determined
+                                normalized.append(AgentWorkerConfig(name=name))
+
+                        case dict() as config:
+                            # If type is explicitly specified, use it
+                            if worker_type := config.get("type"):
+                                match worker_type:
+                                    case "team":
+                                        normalized.append(TeamWorkerConfig(**config))
+                                    case "agent":
+                                        normalized.append(AgentWorkerConfig(**config))
+                                    case _:
+                                        msg = f"Invalid worker type: {worker_type}"
+                                        raise ValueError(msg)
+                            else:
+                                # Determine type based on worker name
+                                worker_name = config.get("name")
+                                if not worker_name:
+                                    msg = "Worker config missing name"
+                                    raise ValueError(msg)
+
+                                if worker_name in teams:
+                                    normalized.append(TeamWorkerConfig(**config))
+                                else:
+                                    normalized.append(AgentWorkerConfig(**config))
+
+                        case BaseWorkerConfig():  # Already normalized
+                            normalized.append(worker)
+
+                        case _:
+                            msg = f"Invalid worker configuration: {worker}"
+                            raise ValueError(msg)
+
+                if isinstance(agent_config, dict):
+                    agent_config["workers"] = normalized
+                else:
+                    # Need to create a new dict with updated workers
+                    agent_dict = agent_config.model_dump()
+                    agent_dict["workers"] = normalized
+                    agents[agent_name] = agent_dict
+
+        return data
+
+    @cached_property
+    def resource_registry(self) -> ResourceRegistry:
+        """Get registry with all configured resources."""
+        registry = ResourceRegistry()
+        for name, config in self.resources.items():
+            if isinstance(config, str):
+                # Convert URI shorthand to SourceResourceConfig
+                config = SourceResourceConfig(uri=config)
+            registry.register_from_config(name, config)
+        return registry
+
     def clone_agent_config(
         self,
         name: str,
@@ -216,7 +321,7 @@ class AgentsManifest(ConfigModel):
     def get_mcp_servers(self) -> list[MCPServerConfig]:
         """Get processed MCP server configurations.
 
-        Converts string entries to StdioMCPServer configs by splitting
+        Converts string entries to StdioMCPServerConfig configs by splitting
         into command and arguments.
 
         Returns:
@@ -235,8 +340,8 @@ class AgentsManifest(ConfigModel):
                         msg = "Empty MCP server command"
                         raise ValueError(msg)
 
-                    configs.append(StdioMCPServer(command=parts[0], args=parts[1:]))
-                case MCPServerBase():
+                    configs.append(StdioMCPServerConfig(command=parts[0], args=parts[1:]))
+                case BaseMCPServerConfig():
                     configs.append(server)
 
         return configs
@@ -305,7 +410,7 @@ class AgentsManifest(ConfigModel):
             description=config.description,
             retries=config.retries,
             session=config.get_session_config(),
-            result_retries=config.result_retries,
+            output_retries=config.output_retries,
             end_strategy=config.end_strategy,
             capabilities=config.capabilities,
             debug=config.debug,
@@ -323,8 +428,6 @@ class AgentsManifest(ConfigModel):
             match agent_config.provider:
                 case "pydantic_ai":
                     providers.add("pydantic_ai")
-                case "litellm":
-                    providers.add("litellm")
                 case BaseProviderConfig():
                     providers.add(agent_config.provider.type)
         return providers
@@ -364,7 +467,7 @@ class AgentsManifest(ConfigModel):
         Returns:
             Configured agent pool
         """
-        from llmling_agent.delegation import AgentPool
+        from llmling_agent import AgentPool
 
         return AgentPool(manifest=self)
 
@@ -379,8 +482,8 @@ class AgentsManifest(ConfigModel):
         logger.debug("Building response model for %r", agent_config.result_type)
         if isinstance(agent_config.result_type, str):
             response_def = self.responses[agent_config.result_type]
-            return response_def.create_model()  # type: ignore
-        return agent_config.result_type.create_model()  # type: ignore
+            return response_def.response_schema.get_schema()  # type: ignore
+        return agent_config.result_type.response_schema.get_schema()  # type: ignore
 
 
 if __name__ == "__main__":
