@@ -31,14 +31,13 @@ def init_agent_config(
     Creates and activates a new agent configuration. The configuration will be
     automatically registered and set as active.
     """
-    # Create config
     from upath import UPath
 
     if interactive:
         validate_import("promptantic", "chat")
         from promptantic import ModelGenerator
 
-        from llmling_agent.models import AgentsManifest
+        from llmling_agent import AgentsManifest
 
         generator = ModelGenerator()
         manifest = generator.populate(AgentsManifest)
@@ -48,7 +47,6 @@ def init_agent_config(
 
         shutil.copy2(config_resources.AGENTS_TEMPLATE, output)
 
-    # Add and set as active
     config_name = name or UPath(output).stem
     agent_store.add_config(config_name, output)
     agent_store.set_active(config_name)
@@ -100,7 +98,7 @@ def list_agents(
     verbose: bool = verbose_opt,
 ):
     """List agents from the active (or specified) configuration."""
-    from llmling_agent.models import AgentsManifest
+    from llmling_agent import AgentsManifest
 
     try:
         try:
@@ -109,7 +107,6 @@ def list_agents(
             msg = str(e)
             raise t.BadParameter(msg) from e
 
-        # Load and validate agent definition
         agent_def = AgentsManifest.from_file(config_path)
         # Set the name field from the dict key for each agent
         agents = [ag.model_copy(update={"name": n}) for n, ag in agent_def.agents.items()]
