@@ -12,7 +12,7 @@ from rich.markdown import Markdown
 from slashed import DefaultOutputWriter, ExitCommandError
 from slashed.prompt_toolkit_completer import PromptToolkitCompleter
 
-from llmling_agent.messaging.messages import ChatMessage
+from llmling_agent import ChatMessage
 from llmling_agent.observability import track_action
 from llmling_agent_cli.chat_session.base import AgentPoolView
 from llmling_agent_cli.chat_session.formatting import MessageFormatter
@@ -21,12 +21,8 @@ from llmling_agent_cli.chat_session.welcome import create_welcome_messages
 
 
 if TYPE_CHECKING:
-    from llmling_agent import Agent
-    from llmling_agent.agent import AnyAgent
+    from llmling_agent import Agent, AgentPool, AnyAgent, Tool, ToolCallInfo
     from llmling_agent.agent.conversation import ConversationManager
-    from llmling_agent.delegation.pool import AgentPool
-    from llmling_agent.models.tools import ToolCallInfo
-    from llmling_agent.tools.base import Tool
 
 
 class InteractiveSession:
@@ -245,11 +241,10 @@ class InteractiveSession:
 
 # Helper function for CLI
 async def start_interactive_session(
-    agent: Agent[Any],
+    agent: AnyAgent[Any, Any],
     *,
-    pool: AgentPool | None = None,
     stream: bool = False,
 ):
     """Start an interactive chat session."""
-    session = InteractiveSession(agent, pool=pool, stream=stream)
+    session = InteractiveSession(agent, pool=agent.context.pool, stream=stream)
     await session.start()
