@@ -2,16 +2,18 @@
 
 from __future__ import annotations
 
-from datetime import datetime
+from datetime import datetime  # noqa: TC003
 import logging
 
 from llmling.cli.constants import output_format_opt
 from llmling.cli.utils import format_output
 import typer as t
 
-from llmling_agent.models import AgentsManifest
+from llmling_agent import AgentsManifest
+from llmling_agent.utils.now import get_now
 from llmling_agent.utils.parse_time import parse_time_period
 from llmling_agent_cli import resolve_agent_config
+from llmling_agent_cli.cli_types import GroupBy  # noqa: TC001
 
 
 logger = logging.getLogger(__name__)
@@ -74,8 +76,8 @@ def show_history(
         # Search for specific content
         llmling-agent history show --query "database schema"
 
-        # Show GPT-4 conversations with token usage
-        llmling-agent history show --model gpt-4 --tokens
+        # Show GPT-5 conversations with token usage
+        llmling-agent history show --model gpt-5 --tokens
 
         # Compact view of recent conversations
         llmling-agent history show --period 1d --compact
@@ -111,9 +113,7 @@ def show_stats(
     period: str = t.Option(
         "1d", "--period", "-p", help="Time period (1h, 1d, 1w, 1m, 1y)"
     ),
-    group_by: str = t.Option(
-        "agent", "--group-by", "-g", help="Group by: agent, model, hour, day"
-    ),
+    group_by: GroupBy = t.Option("agent", "--group-by", "-g", help="Group by"),  # noqa: B008
     output_format: str = output_format_opt,
 ):
     """Show usage statistics.
@@ -137,7 +137,7 @@ def show_stats(
         provider = get_history_provider(config_path)
 
         # Create filters
-        cutoff = datetime.now() - parse_time_period(period)
+        cutoff = get_now() - parse_time_period(period)
         filters = StatsFilters(cutoff=cutoff, group_by=group_by, agent_name=agent_name)  # type: ignore
 
         stats = provider.run_task_sync(provider.get_conversation_stats(filters))
@@ -186,10 +186,8 @@ def reset_history(
             if input(msg).lower() != "y":
                 print("Operation cancelled.")
                 return
-
-        conv_count, msg_count = provider.run_task_sync(
-            provider.reset(agent_name=agent_name, hard=hard)
-        )
+        coro = provider.reset(agent_name=agent_name, hard=hard)
+        conv_count, msg_count = provider.run_task_sync(coro)
 
         what = f" for {agent_name}" if agent_name else ""
         print(f"Deleted {conv_count} conversations and {msg_count} messages{what}.")
