@@ -10,12 +10,13 @@ from tempfile import NamedTemporaryFile
 
 import typer as t
 
-from llmling_agent.delegation import AgentPool
+from llmling_agent import AgentPool
 from llmling_agent.log import set_handler_level
+from llmling_agent_cli.cli_types import LogLevel  # noqa: TC001
 
 
 THEME_HELP = "UI theme (soft/base/monochrome/glass/default)"
-MODEL_HELP = "Model to use (e.g. openai:gpt-4o-mini, gpt-4)"
+MODEL_HELP = "Model to use (e.g. openai:gpt-5-mini)"
 
 QUICKSTART_CONFIG = """\
 agents:
@@ -24,15 +25,13 @@ agents:
         model: {model}
 """
 
-LOG_HELP = "Log level (DEBUG, INFO, WARNING, ERROR)"
+LOG_HELP = "Log level"
 STREAM_HELP = "Enable streaming mode (default: off)"
 
 
 def quickstart_command(
-    model: str = t.Argument("openai:gpt-4o-mini", help=MODEL_HELP),
-    log_level: str = t.Option(
-        "WARNING", "--log-level", "-l", help=LOG_HELP, case_sensitive=False
-    ),
+    model: str = t.Argument("openai:gpt-5-mini", help=MODEL_HELP),
+    log_level: LogLevel = t.Option("warning", "--log-level", "-l", help=LOG_HELP),  # noqa: B008
     stream: bool = t.Option(True, "--stream/--no-stream", help=STREAM_HELP),
 ):
     """Start an ephemeral chat session with minimal setup."""
@@ -51,7 +50,7 @@ def quickstart_command(
         async def run_chat():
             async with AgentPool[None](agent_path) as pool:
                 agent = pool.get_agent("quickstart")
-                await start_interactive_session(agent, pool=pool, stream=stream)
+                await start_interactive_session(agent, stream=stream)
 
         show_logs = False
         output = DefaultOutputWriter() if show_logs else None
