@@ -9,10 +9,11 @@ from typing import TYPE_CHECKING
 import typer as t
 
 from llmling_agent_cli import resolve_agent_config
+from llmling_agent_cli.cli_types import LogLevel  # noqa: TC001
 
 
 if TYPE_CHECKING:
-    from llmling_agent.models import AgentsManifest
+    from llmling_agent import AgentsManifest
 
 
 TASK_HELP = """
@@ -32,7 +33,7 @@ async def execute_job(
     prompt: str | None = None,
 ):
     """Execute task with agent."""
-    from llmling_agent.delegation import AgentPool
+    from llmling_agent import AgentPool
 
     async with AgentPool[None](config) as pool:
         # Get both agent and task
@@ -55,17 +56,15 @@ def task_command(
         None, "--config", "-c", help="Agent configuration file"
     ),
     prompt: str | None = t.Option(None, "--prompt", "-p", help="Additional prompt"),
-    log_level: str = t.Option(
-        "WARNING",
+    log_level: LogLevel = t.Option(  # noqa: B008
+        "warning",
         "--log-level",
         "-l",
-        help="Log level (DEBUG, INFO, WARNING, ERROR)",
-        case_sensitive=False,
+        help="Log level",
     ),
 ):
     """Execute a task with the specified agent."""
     try:
-        # Set up logging
         level = getattr(logging, log_level.upper())
         logging.basicConfig(level=level)
         logger = logging.getLogger("llmling_agent.task")
@@ -79,12 +78,9 @@ def task_command(
             msg = str(e)
             raise t.BadParameter(msg) from e
 
-        # Load manifest
-        from llmling_agent.models import AgentsManifest
+        from llmling_agent import AgentsManifest
 
         manifest = AgentsManifest.from_file(config_path)
-
-        # Execute task
         result = asyncio.run(execute_job(agent_name, task_name, manifest, prompt=prompt))
         print(result)
 
