@@ -1,8 +1,10 @@
+"""Agent session slash commands."""
+
 from __future__ import annotations
 
 from typing import TYPE_CHECKING
 
-from slashed import Command, CommandContext
+from slashed import Command, CommandContext  # noqa: TC002
 
 
 if TYPE_CHECKING:
