@@ -6,7 +6,7 @@ import importlib.util
 from typing import TYPE_CHECKING
 import webbrowser
 
-from slashed import CommandContext, CommandError, SlashedCommand
+from slashed import CommandContext, CommandError, SlashedCommand  # noqa: TC002
 
 
 if TYPE_CHECKING:
@@ -107,9 +107,8 @@ class EditAgentFileCommand(SlashedCommand):
 
         try:
             webbrowser.open(config.config_file_path)
-            await ctx.output.print(
-                f"Opening agent configuration: {config.config_file_path}"
-            )
+            msg = f"Opening agent configuration: {config.config_file_path}"
+            await ctx.output.print(msg)
         except Exception as e:
             msg = f"Failed to open configuration file: {e}"
             raise CommandError(msg) from e
