@@ -4,7 +4,7 @@ from __future__ import annotations
 
 from typing import TYPE_CHECKING
 
-from slashed import Command, CommandContext, CommandError
+from slashed import Command, CommandContext, CommandError  # noqa: TC002
 from slashed.completers import CallbackCompleter
 
 from llmling_agent.log import get_logger
@@ -138,9 +138,7 @@ async def list_workers_command(
 ):
     """List all worker tools."""
     # Filter tools by source="agent"
-    worker_tools = [
-        info for info in ctx.context.agent.tools.values() if info.source == "agent"
-    ]
+    worker_tools = [i for i in ctx.context.agent.tools.values() if i.source == "agent"]
 
     if not worker_tools:
         await ctx.output.print("No worker tools registered")
