@@ -18,10 +18,6 @@ R = TypeVar("R")
 class ObservabilityProvider(ABC):
     """Base class for observability providers."""
 
-    def wrap_agent[T](self, kls: type[T], name: str) -> type[T]:
-        """Wrap an agent class with observability."""
-        return kls
-
     @abstractmethod
     def wrap_tool[T](self, func: Callable[..., T], name: str) -> Callable[..., T]:
         """Wrap a tool function with observability."""
