@@ -4,33 +4,34 @@ from __future__ import annotations
 
 from contextlib import asynccontextmanager
 from dataclasses import asdict
+from decimal import Decimal
 from functools import wraps
 from typing import TYPE_CHECKING, Any, cast, get_args
 
 from llmling_models import AllModels, infer_model
 from pydantic_ai import Agent as PydanticAgent
-import pydantic_ai._pydantic
+import pydantic_ai._function_schema
 from pydantic_ai.messages import ModelResponse
 from pydantic_ai.models import KnownModelName, Model
-from pydantic_ai.result import RunResult, StreamedRunResult
-from pydantic_ai.tools import RunContext
+from pydantic_ai.result import StreamedRunResult
+from pydantic_ai.run import AgentRun
+from pydantic_ai.tools import GenerateToolJsonSchema, RunContext
 from pydantic_ai.usage import UsageLimits as PydanticAiUsageLimits
 
 from llmling_agent.agent.context import AgentContext
-from llmling_agent.common_types import EndStrategy, ModelProtocol
+from llmling_agent.common_types import ModelProtocol
 from llmling_agent.log import get_logger
-from llmling_agent.messaging.messages import ChatMessage, TokenCost
-from llmling_agent.models.content import BaseContent
+from llmling_agent.messaging.messages import TokenCost, TokenUsage
 from llmling_agent.observability import track_action
-from llmling_agent.prompts.convert import format_prompts
 from llmling_agent.tasks.exceptions import (
     ChainAbortedError,
     RunAbortedError,
     ToolSkippedError,
 )
 from llmling_agent.utils.inspection import execute, has_argument_type
-from llmling_agent_providers.base import AgentLLMProvider, ProviderResponse, UsageLimits
+from llmling_agent_providers.base import AgentLLMProvider, ProviderResponse
 from llmling_agent_providers.pydanticai.utils import (
+    convert_prompts_to_user_content,
     format_part,
     get_tool_calls,
     to_model_message,
@@ -40,9 +41,13 @@ from llmling_agent_providers.pydanticai.utils import (
 if TYPE_CHECKING:
     from collections.abc import AsyncIterator, Awaitable, Callable
 
-    from llmling_agent.common_types import ModelType
+    from pydantic_ai.agent import AgentRunResult
+
+    from llmling_agent.common_types import EndStrategy, ModelType
+    from llmling_agent.messaging.messages import ChatMessage
     from llmling_agent.models.content import Content
     from llmling_agent.tools.base import Tool
+    from llmling_agent_providers.base import UsageLimits
 
 
 logger = get_logger(__name__)
@@ -75,10 +80,10 @@ def _is_call_ctx(annotation: Any) -> bool:
     )
 
 
-pydantic_ai._pydantic._is_call_ctx = _is_call_ctx
+pydantic_ai._function_schema._is_call_ctx = _is_call_ctx  # type: ignore
 
 
-class PydanticAIProvider(AgentLLMProvider):
+class PydanticAIProvider[TDeps](AgentLLMProvider[TDeps]):
     """Provider using pydantic-ai as backend."""
 
     NAME = "pydantic_ai"
@@ -90,7 +95,7 @@ class PydanticAIProvider(AgentLLMProvider):
         context: AgentContext | None = None,
         name: str = "agent",
         retries: int = 1,
-        result_retries: int | None = None,
+        output_retries: int | None = None,
         end_strategy: EndStrategy = "early",
         defer_model_check: bool = False,
         debug: bool = False,
@@ -103,7 +108,7 @@ class PydanticAIProvider(AgentLLMProvider):
             context: Agent context
             name: Agent name
             retries: Number of retries for failed operations
-            result_retries: Max retries for result validation
+            output_retries: Max retries for result validation
             end_strategy: How to handle tool calls with final result
             defer_model_check: Whether to defer model validation
             debug: Whether to enable debug mode
@@ -121,7 +126,7 @@ class PydanticAIProvider(AgentLLMProvider):
             tools=[],
             retries=retries,
             end_strategy=end_strategy,
-            result_retries=result_retries,
+            output_retries=output_retries,
             defer_model_check=defer_model_check,
         )
 
@@ -137,7 +142,7 @@ class PydanticAIProvider(AgentLLMProvider):
         kwargs = self._kwargs.copy()
         model = kwargs.pop("model", None)
         model = infer_model(model) if isinstance(model, str) else model
-        agent = PydanticAgent(model=model, system_prompt=system_prompt, **kwargs)  # type: ignore
+        agent = PydanticAgent(model=model, instructions=system_prompt, **kwargs)  # type: ignore
         for tool in tools:
             wrapped = (
                 self.wrap_tool(tool, self._context)
@@ -147,7 +152,17 @@ class PydanticAIProvider(AgentLLMProvider):
             if has_argument_type(wrapped, RunContext):
                 agent.tool(wrapped)
             elif has_argument_type(wrapped, AgentContext):
-                agent._register_function(wrapped, True, 1, None, "auto", False)
+                agent._function_toolset.add_function(
+                    wrapped,
+                    True,
+                    None,
+                    1,
+                    None,
+                    "auto",
+                    False,
+                    GenerateToolJsonSchema,
+                    None,
+                )
             else:
                 agent.tool_plain(wrapped)
         return agent
@@ -250,29 +265,19 @@ class PydanticAIProvider(AgentLLMProvider):
             use_model = infer_model(use_model)
             self.model_changed.emit(use_model)
         try:
-            text_prompts = [p for p in prompts if isinstance(p, str)]
-            content_prompts = [p for p in prompts if isinstance(p, BaseContent)]
-
-            # Get normal text prompt
-            prompt = await format_prompts(text_prompts)
-
-            # Convert Content objects to ModelMessages
-            if content_prompts:
-                prompts_msgs = [
-                    ChatMessage(role="user", content=p) for p in content_prompts
-                ]
-                message_history = [*message_history, *prompts_msgs]
+            # Convert prompts to pydantic-ai format
+            converted_prompts = await convert_prompts_to_user_content(prompts)
 
             # Run with complete history
             to_use = model or self.model
             to_use = infer_model(to_use) if isinstance(to_use, str) else to_use
             limits = asdict(usage_limits) if usage_limits else {}
-            result: RunResult = await agent.run(
-                prompt,
+            result: AgentRunResult = await agent.run(
+                converted_prompts,  # Pass converted prompts
                 deps=self._context,  # type: ignore
                 message_history=[to_model_message(m) for m in message_history],
                 model=to_use,  # type: ignore
-                result_type=result_type or str,
+                output_type=result_type or str,
                 model_settings=self.model_settings,  # type: ignore
                 usage_limits=PydanticAiUsageLimits(**limits),
             )
@@ -284,20 +289,25 @@ class PydanticAIProvider(AgentLLMProvider):
             for call in tool_calls:
                 call.message_id = message_id
                 call.context_data = self._context.data if self._context else None
+                self.tool_used.emit(call)
+
             resolved_model = (
                 use_model.model_name if isinstance(use_model, Model) else str(use_model)
             )
             usage = result.usage()
-            cost_str = prompt + str(content_prompts)  # dirty
-            cost_info = (
-                await TokenCost.from_usage(
-                    usage, resolved_model, cost_str, str(result.data)
-                )
-                if resolved_model and usage
-                else None
+            responses = [m for m in new_msgs if isinstance(m, ModelResponse)]
+            try:
+                cost_sum = sum(i.cost().total_price for i in responses)
+            except LookupError:
+                cost_sum = Decimal(0)
+            token_usage = TokenUsage(
+                total=usage.total_tokens,
+                prompt=usage.input_tokens,
+                completion=usage.output_tokens,
             )
+            cost_info = TokenCost(token_usage=token_usage, total_cost=Decimal(cost_sum))
             return ProviderResponse(
-                content=result.data,
+                content=result.output,
                 tool_calls=tool_calls,
                 cost_and_usage=cost_info,
                 model_name=resolved_model,
@@ -349,6 +359,68 @@ class PydanticAIProvider(AgentLLMProvider):
                 msg = f"Invalid model type: {model}"
                 raise ValueError(msg)
 
+    @asynccontextmanager
+    async def iterate_run[TResult](
+        self,
+        *prompts: str | Content,  # Provider signature
+        message_id: str,
+        message_history: list[ChatMessage[Any]],  # Use Any generic
+        tools: list[Tool] | None = None,
+        result_type: type[TResult] | None = None,
+        usage_limits: UsageLimits | None = None,
+        model: ModelType = None,
+        system_prompt: str | None = None,
+        **kwargs: Any,  # Analogous kwargs
+    ) -> AsyncIterator[AgentRun[TDeps, TResult]]:  # Use Any generic for now
+        """Starts an iterable agent run using pydantic-ai's Agent.iter."""
+        agent = await self.get_agent(system_prompt or "", tools=tools or [])
+        use_model = model or self.model
+        original_model_signal = None
+        if model:
+            original_model_signal = self.model
+            if isinstance(use_model, str):
+                use_model = infer_model(use_model)
+            # Don't emit model_changed here, agent.iter takes model directly
+            # self.model_changed.emit(use_model) # NO EMIT NEEDED HERE
+        elif isinstance(use_model, str):
+            use_model = infer_model(use_model)
+
+        limits = asdict(usage_limits) if usage_limits else {}
+        pydantic_limits = PydanticAiUsageLimits(**limits)
+        resolved_model_name = (
+            use_model.model_name if isinstance(use_model, Model) else str(use_model)
+        )
+
+        try:
+            converted_prompts = await convert_prompts_to_user_content(prompts)
+            model_messages = [to_model_message(m) for m in message_history]
+            msg = "Starting PydanticAI agent iteration run_id=%s with model=%s"
+            logger.debug(msg, message_id, resolved_model_name)
+            async with agent.iter(
+                converted_prompts,
+                deps=self._context,
+                message_history=model_messages,
+                model=model or self.model,  # type: ignore
+                output_type=result_type or str,
+                model_settings=self.model_settings,  # type: ignore
+                usage_limits=pydantic_limits,
+                **kwargs,
+            ) as agent_run:
+                # 7. Post-processing (Minimal for iter, just yield)
+                # Add model name to the run object if possible/needed?
+                # setattr(agent_run, 'model_name', resolved_model_name)
+
+                yield cast(AgentRun[Any, Any], agent_run)  # Yield the run object
+        except Exception as e:
+            # Catch potential errors from argument mismatches in agent.iter
+            logger.exception("Error calling agent.iter")
+            msg = f"Iteration setup error: {e}"
+            raise RuntimeError(msg) from e
+        finally:
+            if original_model_signal and isinstance(original_model_signal, str):
+                original_model_signal = infer_model(original_model_signal)
+            # self.model_changed.emit(original_model_signal)
+
     @asynccontextmanager
     async def stream_response(  # type: ignore[override]
         self,
@@ -372,63 +444,61 @@ class PydanticAIProvider(AgentLLMProvider):
         if model:
             self.model_changed.emit(use_model)
 
-        text_prompts = [p for p in prompts if isinstance(p, str)]
-        content_prompts = [p for p in prompts if isinstance(p, BaseContent)]
-
-        # Get normal text prompt
-        prompt = await format_prompts(text_prompts)
-
-        # Convert Content objects to ChatMessages
-        if content_prompts:
-            prompts_msgs = [ChatMessage(role="user", content=p) for p in content_prompts]
-            message_history = [*message_history, *prompts_msgs]
+        # Convert prompts to pydantic-ai format
+        converted_prompts = await convert_prompts_to_user_content(prompts)
 
         # Convert all messages to pydantic-ai format
         model_messages = [to_model_message(m) for m in message_history]
 
         async with agent.run_stream(
-            prompt,
+            converted_prompts,
             deps=self._context,
             message_history=model_messages,
             model=model or self.model,  # type: ignore
-            result_type=result_type or str,
+            output_type=result_type or str,
             model_settings=self.model_settings,  # type: ignore
             usage_limits=PydanticAiUsageLimits(**limits),
         ) as stream_result:
             stream_result = cast(StreamedRunResult[AgentContext[Any], Any], stream_result)
-            original_stream = stream_result.stream
+            original_stream = stream_result.stream_output
+            original_text_stream = stream_result.stream_text
+            resolved_model = (
+                use_model.model_name if isinstance(use_model, Model) else str(use_model)
+            )
+            stream_result.model_name = resolved_model  # type: ignore
 
-            async def wrapped_stream(*args, **kwargs):
-                last_content = None
-                async for chunk in original_stream(*args, **kwargs):
-                    # Only emit if content has changed
-                    if chunk != last_content:
-                        self.chunk_streamed.emit(str(chunk), message_id)
-                        last_content = chunk
-                        yield chunk
+            def get_wrapped_stream(fn):
+                async def wrapped_stream(*args, **kwargs):
+                    last_content = None
+                    async for chunk in fn(*args, **kwargs):
+                        # Only emit if content has changed
+                        if chunk != last_content:
+                            self.chunk_streamed.emit(str(chunk), message_id)
+                            last_content = chunk
+                            yield chunk
 
-                if stream_result.is_complete:
-                    self.chunk_streamed.emit("", message_id)
-                    messages = stream_result.new_messages()
-                    tool_dict = {i.name: i for i in tools or []}
-                    # Extract and update tool calls
-                    tool_calls = get_tool_calls(messages, tool_dict, agent_name=self.name)
-                    for call in tool_calls:
-                        call.message_id = message_id
-                        call.context_data = self._context.data if self._context else None
+                    if stream_result.is_complete:
+                        self.chunk_streamed.emit("", message_id)
+                        messages = stream_result.new_messages()
+                        tool_dict = {i.name: i for i in tools or []}
+                        # Extract and update tool calls
+                        tool_calls = get_tool_calls(
+                            messages, tool_dict, agent_name=self.name
+                        )
+                        for call in tool_calls:
+                            call.message_id = message_id
+                            call.context_data = (
+                                self._context.data if self._context else None
+                            )
+                            self.tool_used.emit(call)
+                        # Format final content
+                        responses = [m for m in messages if isinstance(m, ModelResponse)]
+                        parts = [p for msg in responses for p in msg.parts]
+                        content = "\n".join(format_part(p) for p in parts)
+                        # Update stream result with formatted content
+                        stream_result.formatted_content = content  # type: ignore
 
-                    # Format final content
-                    responses = [m for m in messages if isinstance(m, ModelResponse)]
-                    parts = [p for msg in responses for p in msg.parts]
-                    content = "\n".join(format_part(p) for p in parts)
-                    resolved_model = (
-                        use_model.model_name
-                        if isinstance(use_model, Model)
-                        else str(use_model)
-                    )
-                    # Update stream result with formatted content
-                    stream_result.formatted_content = content  # type: ignore
-                    stream_result.model_name = resolved_model  # type: ignore
+                return wrapped_stream
 
             if model:
                 original = self.model
@@ -436,5 +506,23 @@ class PydanticAIProvider(AgentLLMProvider):
                     original = infer_model(original)
                 self.model_changed.emit(original)
 
-            stream_result.stream = wrapped_stream  # type: ignore
+            stream_result.stream_output = get_wrapped_stream(original_stream)  # type: ignore
+            stream_result.stream_text = get_wrapped_stream(original_text_stream)  # type: ignore
             yield stream_result  # type: ignore
+
+
+if __name__ == "__main__":
+    import asyncio
+
+    async def main():
+        provider = PydanticAIProvider[None](model="openai:gpt-5-nano")
+
+        # Example usage
+        response = await provider.generate_response(
+            "Hello, how are you?",
+            message_id="test",
+            message_history=[],
+        )
+        print(response.cost_and_usage)
+
+    asyncio.run(main())
