@@ -3,6 +3,7 @@
 from __future__ import annotations
 
 from datetime import datetime
+from decimal import Decimal
 from typing import TYPE_CHECKING, Any
 
 from sqlalchemy import JSON, Column, and_, or_
@@ -20,7 +21,7 @@ if TYPE_CHECKING:
     from sqlmodel.sql.expression import SelectOfScalar
     from tokonomics.toko_types import TokenUsage
 
-    from llmling_agent.models.session import SessionQuery
+    from llmling_agent_config.session import SessionQuery
     from llmling_agent_storage.sql_provider.models import Message
 
 
@@ -34,8 +35,8 @@ def aggregate_token_usage(
     for msg in messages:
         if isinstance(msg, Message):
             total += msg.total_tokens or 0
-            prompt += msg.prompt_tokens or 0
-            completion += msg.completion_tokens or 0
+            prompt += msg.input_tokens or 0
+            completion += msg.output_tokens or 0
         elif msg.cost_info:
             total += msg.cost_info.token_usage.get("total", 0)
             prompt += msg.cost_info.token_usage.get("prompt", 0)
@@ -50,13 +51,15 @@ def to_chat_message(db_message: Message) -> ChatMessage[str]:
         cost_info = TokenCost(
             token_usage={
                 "total": db_message.total_tokens or 0,
-                "prompt": db_message.prompt_tokens or 0,
-                "completion": db_message.completion_tokens or 0,
+                "prompt": db_message.input_tokens or 0,
+                "completion": db_message.output_tokens or 0,
             },
-            total_cost=db_message.cost or 0.0,
+            total_cost=Decimal(db_message.cost or 0.0),
         )
 
     return ChatMessage[str](
+        message_id=db_message.id,
+        conversation_id=db_message.conversation_id,
         content=db_message.content,
         role=db_message.role,  # type: ignore
         name=db_message.name,
@@ -64,6 +67,7 @@ def to_chat_message(db_message: Message) -> ChatMessage[str]:
         cost_info=cost_info,
         response_time=db_message.response_time,
         forwarded_from=db_message.forwarded_from or [],
+        timestamp=db_message.timestamp,
     )
 
 
@@ -84,7 +88,7 @@ def parse_model_info(model: str | None) -> tuple[str | None, str | None]:
     """Parse model string into provider and name.
 
     Args:
-        model: Full model string (e.g., "openai:gpt-4", "anthropic/claude-2")
+        model: Full model string (e.g., "openai:gpt-5", "anthropic:claude-sonnet-4-0")
 
     Returns:
         Tuple of (provider, name)
@@ -192,7 +196,7 @@ def format_conversation(
                 "model": msg.model,
                 "name": msg.name,
                 "token_usage": msg.cost_info.token_usage if msg.cost_info else None,
-                "cost": msg.cost_info.total_cost if msg.cost_info else None,
+                "cost": float(msg.cost_info.total_cost) if msg.cost_info else None,
                 "response_time": msg.response_time,
             }
             for msg in chat_messages
