@@ -1,8 +1,10 @@
+"""Capability tools."""
+
 from __future__ import annotations
 
 import asyncio
 import contextlib
-from datetime import datetime, timedelta
+from datetime import timedelta
 import io
 from typing import TYPE_CHECKING, Any, Literal
 from uuid import uuid4
@@ -10,10 +12,15 @@ from uuid import uuid4
 from llmling import ToolError
 
 from llmling_agent.agent.context import AgentContext  # noqa: TC001
+from llmling_agent.log import get_logger
+from llmling_agent.utils.now import get_now
 
 
 if TYPE_CHECKING:
     from llmling_agent.agent import AnyAgent
+    from llmling_agent.common_types import StrPath
+
+logger = get_logger(__name__)
 
 
 async def delegate_to(  # noqa: D417
@@ -242,7 +249,7 @@ async def show_statistics(
 
     if isinstance(ctx, RunContext):
         ctx = ctx.deps
-    cutoff = datetime.now() - timedelta(hours=hours)
+    cutoff = get_now() - timedelta(hours=hours)
     filters = StatsFilters(cutoff=cutoff, group_by=group_by)
 
     provider = ctx.storage.get_history_provider()
@@ -281,19 +288,49 @@ async def execute_python(ctx: AgentContext, code: str) -> str:
         return f"Error executing code: {e}"
 
 
-async def execute_command(ctx: AgentContext, command: str) -> str:
-    """Execute a shell command."""
+async def execute_command(  # noqa: D417
+    ctx: AgentContext,
+    command: str,
+    env: dict[str, str] | None = None,
+    output_limit: int | None = None,
+) -> str:
+    """Execute a shell command.
+
+    Args:
+        command: Shell command to execute
+        env: Environment variables to add to current environment
+        output_limit: Maximum bytes of output to return
+    """
+    import os
+
     from pydantic_ai.tools import RunContext
 
     if isinstance(ctx, RunContext):
         ctx = ctx.deps
     try:
+        # Prepare environment
+        proc_env = dict(os.environ)
+        if env:
+            proc_env.update(env)
+
         pipe = asyncio.subprocess.PIPE
-        proc = await asyncio.create_subprocess_shell(command, stdout=pipe, stderr=pipe)
+        proc = await asyncio.create_subprocess_shell(
+            command, stdout=pipe, stderr=pipe, env=proc_env
+        )
         stdout, stderr = await proc.communicate()
-        return stdout.decode() or stderr.decode() or "Command completed"
+
+        # Combine and decode output
+        output = stdout.decode() or stderr.decode() or "Command completed"
+
+        # Apply output limit if specified
+        if output_limit and len(output.encode()) > output_limit:
+            # Truncate from the end to keep most recent output
+            truncated_output = output.encode()[-output_limit:].decode(errors="ignore")
+            output = f"...[truncated]\n{truncated_output}"
     except Exception as e:  # noqa: BLE001
         return f"Error executing command: {e}"
+    else:
+        return output
 
 
 async def add_agent(  # noqa: D417
@@ -395,11 +432,7 @@ async def ask_agent(  # noqa: D417
         ctx = ctx.deps
     assert ctx.pool, "No agent pool available"
     agent = ctx.pool.get_agent(agent_name)
-    result = await agent.run(
-        message,
-        model=model,
-        store_history=store_history,
-    )
+    result = await agent.run(message, model=model, store_history=store_history)
     return str(result.content)
 
 
@@ -479,3 +512,276 @@ async def connect_nodes(  # noqa: D417
         f"(type={connection_type}, queued={queued}, "
         f"strategy={queue_strategy if queued else 'n/a'})"
     )
+
+
+async def read_file(  # noqa: D417
+    ctx: AgentContext,
+    path: str,
+    *,
+    convert_to_markdown: bool = True,
+    encoding: str = "utf-8",
+    line: int | None = None,
+    limit: int | None = None,
+) -> str:
+    """Read file content from local or remote path.
+
+    Args:
+        path: Path or URL to read
+        convert_to_markdown: Whether to convert content to markdown
+        encoding: Text encoding to use (default: utf-8)
+        line: Optional line number to start reading from (1-based)
+        limit: Optional maximum number of lines to read
+
+    Returns:
+        File content, optionally converted to markdown
+    """
+    from pydantic_ai.tools import RunContext
+    from upathtools import read_path
+
+    if isinstance(ctx, RunContext):
+        ctx = ctx.deps
+
+    try:
+        # First try to read raw content
+        content = await read_path(path, encoding=encoding)
+
+        # Convert to markdown if requested
+        if convert_to_markdown and ctx.converter:
+            try:
+                content = await ctx.converter.convert_file(path)
+            except Exception as e:  # noqa: BLE001
+                msg = f"Failed to convert to markdown: {e}"
+                logger.warning(msg)
+                # Continue with raw content
+
+        # Apply line filtering if requested
+        if line is not None or limit is not None:
+            lines = content.splitlines(keepends=True)
+            start_idx = (line - 1) if line is not None else 0
+            end_idx = start_idx + limit if limit is not None else len(lines)
+            content = "".join(lines[start_idx:end_idx])
+
+    except Exception as e:
+        msg = f"Failed to read file {path}: {e}"
+        raise ToolError(msg) from e
+    else:
+        return content
+
+
+async def list_directory(
+    path: StrPath,
+    *,
+    pattern: str | None = None,
+    recursive: bool = True,
+    include_dirs: bool = False,
+    exclude: list[str] | None = None,
+    max_depth: int | None = None,
+) -> str:
+    """List files / subfolders in a folder.
+
+    Args:
+        path: Base directory to read from
+        pattern: Glob pattern to match files against (e.g. "**/*.py" for Python files)
+        recursive: Whether to search subdirectories
+        include_dirs: Whether to include directories in results
+        exclude: List of patterns to exclude (uses fnmatch against relative paths)
+        max_depth: Maximum directory depth for recursive search
+
+    Returns:
+        A list of files / folders.
+    """
+    from upathtools import list_files
+
+    pattern = pattern or "**/*"
+    files = await list_files(
+        path,
+        pattern=pattern,
+        include_dirs=include_dirs,
+        recursive=recursive,
+        exclude=exclude,
+        max_depth=max_depth,
+    )
+    return "\n".join(str(f) for f in files)
+
+
+async def start_process(  # noqa: D417
+    ctx: AgentContext,
+    command: str,
+    args: list[str] | None = None,
+    cwd: str | None = None,
+    env: dict[str, str] | None = None,
+    output_limit: int | None = None,
+) -> str:
+    """Start a command in the background and return immediately with process ID.
+
+    Args:
+        command: Command to execute
+        args: Command arguments
+        cwd: Working directory
+        env: Environment variables (added to current env)
+        output_limit: Maximum bytes of output to retain
+
+    Returns:
+        Process ID for tracking the background process
+    """
+    from pydantic_ai.tools import RunContext
+
+    if isinstance(ctx, RunContext):
+        ctx = ctx.deps
+
+    try:
+        process_id = await ctx.process_manager.start_process(
+            command=command,
+            args=args,
+            cwd=cwd,
+            env=env,
+            output_limit=output_limit,
+        )
+    except Exception as e:  # noqa: BLE001
+        return f"Failed to start process: {e}"
+    else:
+        return f"Started process: {process_id}"
+
+
+async def get_process_output(ctx: AgentContext, process_id: str) -> str:  # noqa: D417
+    """Get current output from a background process.
+
+    Args:
+        process_id: Process identifier from start_background_process
+
+    Returns:
+        Current process output (stdout + stderr)
+    """
+    from pydantic_ai.tools import RunContext
+
+    if isinstance(ctx, RunContext):
+        ctx = ctx.deps
+
+    try:
+        output = await ctx.process_manager.get_output(process_id)
+        result = f"Process {process_id}:\n"
+        if output.stdout:
+            result += f"STDOUT:\n{output.stdout}\n"
+        if output.stderr:
+            result += f"STDERR:\n{output.stderr}\n"
+        if output.exit_code is not None:
+            result += f"Exit code: {output.exit_code}\n"
+        if output.truncated:
+            result += "Note: Output was truncated due to size limits\n"
+        return result.strip()
+    except ValueError as e:
+        return str(e)
+    except Exception as e:  # noqa: BLE001
+        return f"Error getting process output: {e}"
+
+
+async def wait_for_process(ctx: AgentContext, process_id: str) -> str:  # noqa: D417
+    """Wait for background process to complete and return final output.
+
+    Args:
+        process_id: Process identifier from start_process
+
+    Returns:
+        Final process output and exit code
+    """
+    from pydantic_ai.tools import RunContext
+
+    if isinstance(ctx, RunContext):
+        ctx = ctx.deps
+
+    try:
+        exit_code = await ctx.process_manager.wait_for_exit(process_id)
+        output = await ctx.process_manager.get_output(process_id)
+
+        result = f"Process {process_id} completed with exit code {exit_code}\n"
+        if output.stdout:
+            result += f"STDOUT:\n{output.stdout}\n"
+        if output.stderr:
+            result += f"STDERR:\n{output.stderr}\n"
+        if output.truncated:
+            result += "Note: Output was truncated due to size limits\n"
+        return result.strip()
+    except ValueError as e:
+        return str(e)
+    except Exception as e:  # noqa: BLE001
+        return f"Error waiting for process: {e}"
+
+
+async def kill_process(ctx: AgentContext, process_id: str) -> str:  # noqa: D417
+    """Terminate a background process.
+
+    Args:
+        process_id: Process identifier from start_process
+
+    Returns:
+        Confirmation message
+    """
+    from pydantic_ai.tools import RunContext
+
+    if isinstance(ctx, RunContext):
+        ctx = ctx.deps
+
+    try:
+        await ctx.process_manager.kill_process(process_id)
+    except ValueError as e:
+        return str(e)
+    except Exception as e:  # noqa: BLE001
+        return f"Error killing process: {e}"
+    else:
+        return f"Process {process_id} has been terminated"
+
+
+async def release_process(ctx: AgentContext, process_id: str) -> str:  # noqa: D417
+    """Release resources for a background process.
+
+    Args:
+        process_id: Process identifier from start_process
+
+    Returns:
+        Confirmation message
+    """
+    from pydantic_ai.tools import RunContext
+
+    if isinstance(ctx, RunContext):
+        ctx = ctx.deps
+
+    try:
+        await ctx.process_manager.release_process(process_id)
+    except ValueError as e:
+        return str(e)
+    except Exception as e:  # noqa: BLE001
+        return f"Error releasing process: {e}"
+    else:
+        return f"Process {process_id} resources have been released"
+
+
+async def list_processes(ctx: AgentContext) -> str:
+    """List all active background processes.
+
+    Returns:
+        List of process IDs and basic information
+    """
+    from pydantic_ai.tools import RunContext
+
+    if isinstance(ctx, RunContext):
+        ctx = ctx.deps
+
+    try:
+        process_ids = ctx.process_manager.list_processes()
+        if not process_ids:
+            return "No active processes"
+
+        result = "Active processes:\n"
+        for process_id in process_ids:
+            try:
+                info = await ctx.process_manager.get_process_info(process_id)
+                status = (
+                    "running" if info["is_running"] else f"exited ({info['exit_code']})"
+                )
+                args = " ".join(info["args"])
+                result += f"- {process_id}: {info['command']} {args} [{status}]\n"
+            except Exception as e:  # noqa: BLE001
+                result += f"- {process_id}: Error getting info - {e}\n"
+        return result.strip()
+    except Exception as e:  # noqa: BLE001
+        return f"Error listing processes: {e}"
