@@ -8,7 +8,7 @@ from textual.widgets import DataTable
 
 
 if TYPE_CHECKING:
-    from llmling_agent.delegation.pool import AgentPool
+    from llmling_agent import AgentPool
 
 
 class NodeListView(DataTable):
@@ -27,3 +27,9 @@ class NodeListView(DataTable):
             self.add_row(
                 name, status, agent.model_name or "default", ", ".join(connections) or "-"
             )
+
+
+if __name__ == "__main__":
+    from textualicious import show
+
+    show(NodeListView())
